import unittest
from babelvueextractor.lexer import Lexer, Token, TOKEN_TEXT, TOKEN_VAR, TOKEN_COMMENT, TOKEN_RAW_HTML, TOKEN_CONST, \
    TOKEN_DOUBLE_WAY_BINDING


class TestLexer(unittest.TestCase):
    def assertTokensEqual(self, t_a, t_b):
        for t1, t2 in zip(t_a, t_b):
            self.assertEqual(t1, t2)

    def test_text(self):
        content = "<div></div>"
        self.assertTokensEqual(
            Lexer(content).tokenize(), [
                Token(TOKEN_TEXT, '<div></div>')
            ])

    def test_var(self):
        content = "<div>{{ bar }}</div>"
        self.assertTokensEqual(
            Lexer(content).tokenize(), [
                Token(TOKEN_TEXT, '<div>'),
                Token(TOKEN_VAR, "bar"),
                Token(TOKEN_TEXT, '</div>')
            ])

    def test_comments(self):
        content = "<!-- comments -->"
        self.assertTokensEqual(
            Lexer(content).tokenize(), [
                Token(TOKEN_COMMENT, "comments")
            ])

    def test_raw_html(self):
        content = "{{{ <div></div> }}}"
        self.assertTokensEqual(
            Lexer(content).tokenize(), [
                Token(TOKEN_RAW_HTML, "<div></div>")
            ])

    def test_const(self):
        content = "{{* bar }}"
        self.assertTokensEqual(
            Lexer(content).tokenize(), [
                Token(TOKEN_CONST, "bar")
            ])

    def test_double_way_binding(self):
        content = "{{@ foo }}"
        self.assertTokensEqual(
            Lexer(content).tokenize(), [
                Token(TOKEN_DOUBLE_WAY_BINDING, "foo")
            ])

    def test_combine(self):
        content = "<div>{{ gettext('Hello') }}</div>" \
                  "<div>{{* gettext('Hello') }}</div>" \
                  "<div><!-- Hello -->" \
                  "<span>{{{ Blablabla }}}</span></div>"

        self.assertTokensEqual(
            Lexer(content).tokenize(), [
                Token(token_type=0, contents="<div>"),
                Token(token_type=1, contents="gettext('Hello')"),
                Token(token_type=0, contents="</div><div>"),
                Token(token_type=2, contents="gettext('Hello')"),
                Token(token_type=0, contents="</div><div>"),
                Token(token_type=3, contents="Hello"),
                Token(token_type=0, contents="<span>"),
                Token(token_type=4, contents="Blablabla"),
                Token(token_type=0, contents="</span></div>")
            ])

    def test_str(self):
        t = Token(token_type=TOKEN_RAW_HTML, contents='<!DOCTYPE html>' +
              '<html><head><meta http-equiv="Content-Type" content="text/html; charset=utf-8" />' +
              '<title>HTML Document</title></head><body><p><b></b></p></body></html>')
        self.assertEqual('<Raw token: "<!DOCTYPE html><html...">', t.__str__())

    def test_repr(self):
        t = Token(token_type=TOKEN_RAW_HTML, contents='<body><p><b></b></p></body></html>')
        self.assertEqual('Token(token_type=4, contents="<body><p><b></b></p></body></html>")', t.__repr__())

    def test_token_name(self):
        self.assertEqual(Token(token_type=0, contents='').token_name, 'Text')
        self.assertEqual(Token(token_type=1, contents='').token_name, 'Var')
        self.assertEqual(Token(token_type=2, contents='').token_name, 'Const')
        self.assertEqual(Token(token_type=3, contents='').token_name, 'Comment')
        self.assertEqual(Token(token_type=4, contents='').token_name, 'Raw')
        self.assertEqual(Token(token_type=5, contents='').token_name, 'Binding')


from __future__ import unicode_literals
import re

TOKEN_TEXT = 0
TOKEN_VAR = 1
TOKEN_CONST = 2
TOKEN_COMMENT = 3
TOKEN_RAW_HTML = 4
TOKEN_DOUBLE_WAY_BINDING = 5
TOKEN_DIRECTIVE = 6

TOKEN_MAPPING = {
    TOKEN_TEXT: 'Text',
    TOKEN_VAR: 'Var',
    TOKEN_CONST: 'Const',
    TOKEN_DOUBLE_WAY_BINDING: 'Binding',
    TOKEN_COMMENT: 'Comment',
    TOKEN_RAW_HTML: 'Raw',
    TOKEN_DIRECTIVE: 'Directive',
}

# template syntax constants
FILTER_SEPARATOR = '|'
FILTER_ARGUMENT_SEPARATOR = ':'
VARIABLE_ATTRIBUTE_SEPARATOR = '.'

VARIABLE_TAG_START = '{{'
VARIABLE_TAG_END = '}}'
CONST_START = '{{*'
CONST_END = '}}'
RAW_HTML_TAG_START = '{{{'
RAW_HTML_TAG_END = '}}}'
DOUBLE_WAY_BINDING_START = '{{@'
DOUBLE_WAY_BINDING_END = '}}'
COMMENT_START = '<!--'
COMMENT_END = '-->'
DIRECTIVE_START = 'v-text="'
DIRECTIVE_END = '"'

tag_re = re.compile('(%s.*?%s|%s.*?%s|%s.*?%s|%s.*?%s|%s.*?%s|%s.*?%s)' % (
    re.escape(CONST_START), re.escape(CONST_END),
    re.escape(RAW_HTML_TAG_START), re.escape(RAW_HTML_TAG_END),
    re.escape(VARIABLE_TAG_START), re.escape(VARIABLE_TAG_END),
    re.escape(COMMENT_START), re.escape(COMMENT_END),
    re.escape(DOUBLE_WAY_BINDING_START), re.escape(DOUBLE_WAY_BINDING_END),
    re.escape(DIRECTIVE_START), re.escape(DIRECTIVE_END),
))


class Token(object):
    def __init__(self, token_type, contents):
        self.token_type, self.contents = token_type, contents
        self.lineno = None

    def __str__(self):
        token_name = TOKEN_MAPPING[self.token_type]
        return ('<{0} token: "{1}...">'.format(
            token_name, self.contents[:20].replace('\n', '')))

    def __repr__(self):
        return 'Token(token_type={token_type}, contents="{contents}")'.format(
            token_type=self.token_type,
            contents=self.contents
        )

    def __eq__(self, other):
        return all([self.token_type == other.token_type, self.contents == other.contents])

    @property
    def token_name(self):
        return TOKEN_MAPPING[self.token_type]


class Lexer(object):
    def __init__(self, template_string, origin=None):
        self.template_string = template_string
        self.origin = origin
        self.lineno = 1
        self.verbatim = False

    def tokenize(self):
        """
        Return a list of tokens from a given template_string.
        """
        in_tag = False
        result = []
        for bit in tag_re.split(self.template_string):
            if bit:
                result.append(
                    self.create_token(bit, in_tag)
                )
            in_tag = not in_tag
        return result

    def create_token(self, token_string, in_tag):
        """
        Convert the given token string into a new Token object and return it.
        If in_tag is True, we are processing something that matched a tag,
        otherwise it should be treated as a literal string.
        """
        token_type = TOKEN_TEXT
        content = token_string

        if in_tag:

            if DOUBLE_WAY_BINDING_START in token_string:
                token_type = TOKEN_DOUBLE_WAY_BINDING
                content = token_string[3:-2].strip()

            elif CONST_START in token_string:
                token_type = TOKEN_CONST
                content = token_string[3:-2].strip()

            elif RAW_HTML_TAG_START in token_string:
                token_type = TOKEN_RAW_HTML
                content = token_string[3:-3].strip()

            elif COMMENT_START in token_string:
                token_type = TOKEN_COMMENT
                content = token_string[4:-3].strip()

            elif VARIABLE_TAG_START in token_string:
                token_type = TOKEN_VAR
                content = token_string[2:-2].strip()

            elif DIRECTIVE_START in token_string:
                token_type = TOKEN_DIRECTIVE
                content = token_string[8:-1].strip()

        token = Token(token_type, content)
        token.lineno = self.lineno
        self.lineno += token_string.count('\n')
        return token



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest

from babel.messages.extract import DEFAULT_KEYWORDS, extract

from babelvueextractor.extract import extract_vue
from six import StringIO as FileMock

TEST_OPTIONS = {}


class TestMessagesExtractor(unittest.TestCase):
    def test_empty_template(self):
        template = FileMock("")
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], TEST_OPTIONS)
        self.assertEqual(list(), list(result))

    def test_no_messages(self):
        template = FileMock("""
        <div>
            <h1>Foo</h1>
            <p>Lore ipsum delore gettext() ...</p>
        </div>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], TEST_OPTIONS)
        self.assertEqual(list(), list(result))

    def test_gettext(self):
        template = FileMock("""
        <div>
            {{ gettext('Foo') }}
        </div>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], TEST_OPTIONS)
        self.assertEqual(list(result), [
            (3, u'gettext', u"Foo", [])
        ])

    def test_ngettext(self):
        template = FileMock("""
        <div>
            {{ ngettext('Foo', 'Foos', 1) }}
        </div>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], TEST_OPTIONS)
        self.assertEqual(list(result), [
            (3, u'ngettext', (u"Foo", u"Foos"), []),
        ])

    @unittest.skip("Not using underscore because it conflicts with underscore.js")
    def test_underscore(self):
        template = FileMock("""
        <div>
            {{ _("Bar") }}
        </div>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], TEST_OPTIONS)
        self.assertEqual(list(result), [
            (3, '_', u'Bar', [])
        ])

    def test_token_without_content(self):
        template = FileMock("""
            {{  }}
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], TEST_OPTIONS)
        self.assertEqual(list(result), [])

    def test_commas(self):
        template = FileMock("""
            {{ gettext('Hello, User') }}
            {{ gettext("You're") }}
            {{ gettext("You\\"re") }}
        """)

        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], TEST_OPTIONS)
        self.assertEqual(list(result), [
            (2, u'gettext', u'Hello, User', []),
            (3, u'gettext', u"You're", []),
            (4, u'gettext', u"You\"re", []),
        ])

    def test_babel(self):
        method = 'babelvueextractor.extract.extract_vue'
        fileobj = open('babelvueextractor/tests/templates/for_babel.vhtml')
        result = extract(method, fileobj)

        self.assertListEqual(list(result), [
            (1, u'Привет, User', [], None),
            (2, (u'Здравствуй, друг', u'Здравствуйте, друзья'), [], None)
        ])

    def test_gettext_with_parameter(self):
        template = FileMock("""
        <li>
            {{ gettext('{number} season').replace("{number}", season) }}
            {{ gettext('Processed by filter')|somefilter }}
            {{ somefunc(gettext('Processed by function')) }}
            {{ gettext('Foo') + gettext('bar') }}
        </li>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], TEST_OPTIONS)
        self.assertEqual(list(result), [
            (3, u'gettext', u'{number} season', []),
            (4, u'gettext', u'Processed by filter', []),
            (5, u'gettext', u'Processed by function', []),
            (6, u'gettext', u'Foo', []),
            (6, u'gettext', u'bar', []),
        ])

    def test_directives(self):
        template = FileMock("""
        <div v-text="gettext('Sometext')"></div>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], TEST_OPTIONS)
        self.assertListEqual(list(result), [(2, u'gettext', u'Sometext', [])])

    def test_directives_with_inner_tag(self):
        template = FileMock("""
        <div v-text="gettext('Sometext')">
        {{ gettext('Hello') }}
        </div>
        """)
        result = extract_vue(template, DEFAULT_KEYWORDS.keys(), [], TEST_OPTIONS)
        self.assertListEqual(list(result), [
            (2, u'gettext', u'Sometext', []),
            (3, u'gettext', u'Hello', [])
        ])



"""A setuptools based setup module for babel-vue-extractor

Based on:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

PROJECT_DIR = path.dirname(__file__)


def read(*parts):
    filename = path.join(PROJECT_DIR, *parts)
    with open(filename, encoding='utf-8') as fp:
        return fp.read()

REQUIREMENTS_FILE = 'requirements.txt'
REQUIREMENTS = open(path.join(PROJECT_DIR, REQUIREMENTS_FILE)).readlines()

setup(
    name='babel-vue-extractor',
    version=read('VERSION'),
    description='A plugin for babel to work with vue.js templates',
    long_description=read('README.rst'),
    url='https://github.com/nonamenix/babel-vue-extractor',
    author='Danil Ivanov',
    author_email='nonamenix@gmail.com',
    license='Apache Software License',
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Internationalization',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],
    keywords='vuejs babel',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=REQUIREMENTS,
    extras_require={
        'dev': ['check-manifest', 'nose'],
        'test': ['coverage'],
    },
)



from __future__ import unicode_literals
import ast
import functools

import six

from babelvueextractor.lexer import (
    Lexer,
    Token,
    TOKEN_VAR,
    TOKEN_CONST,
    TOKEN_RAW_HTML,
    TOKEN_DOUBLE_WAY_BINDING,
    TOKEN_DIRECTIVE
)

TOKENS = [TOKEN_VAR, TOKEN_CONST, TOKEN_RAW_HTML, TOKEN_DOUBLE_WAY_BINDING, TOKEN_DIRECTIVE]


def extract_vue(fileobj, keywords, comment_tags, options):
    """Extract messages from Vue template files.

    :param fileobj: the file-like the messages should be extracted from
    :param keywords: a list of keywords (i.e. function names) that should be recognize as translation functions
    :param comment_tags: a list of translator tags to search for and include in the results
    :param options: a dictionary of additional options (optional)
    :return: an iterator over ``(lineno, funcname, message, comments)``
    :rtype: ``iterator``
    """
    contents = fileobj.read()
    u = lambda s: s if isinstance(s, six.text_type) else s.decode(encoding=options.get('encoding', 'utf-8'))
    contents = u(contents)
    lexer = Lexer(contents, None)
    for t in lexer.tokenize():  # type: Token
        if t.token_type in TOKENS:
            try:
                tree = ast.parse(t.contents).body[0]
            except (SyntaxError, IndexError, AttributeError):
                pass
            else:
                for node in ast.walk(tree):  # type: ast.Call
                    if (isinstance(node, ast.Call)
                            and isinstance(node.func, ast.Name)):
                        func_name = node.func.id
                        args = node.args
                        messages = None
                        if func_name == "gettext":
                            if len(args) != 1:
                                raise TypeError("Error at line %s: Function gettext()"
                                                " requires exactly one argument" % t.lineno)
                            if isinstance(args[0], ast.Str):  # ignore other argument types such as variables
                                messages = u(args[0].s)
                        elif func_name == "ngettext":
                            if len(node.args) != 3:
                                raise TypeError("Error at line %s: Function ngettext()"
                                                " requires exactly 3 arguments" % t.lineno)
                            if isinstance(args[0], ast.Str) and isinstance(args[1], ast.Str):
                                messages = u(args[0].s), u(args[1].s)

                        if messages:
                            yield t.lineno, func_name, messages, []
